
/*  Speak Freely version number  */

/*  It is *essential* you write the version number as a continuation
    line of the #define Relno statement and that line must be the last
    line of the version.h file.  The code that embeds the version
    number in the sfvod Perl script assumes this is the case and will
    die a horrible and difficult to diagnose death if it isn't so.  */

#define Relno   \
"Release 6.1c, September 1997"
